/**
 * @module
 * @description
 * Maps application URLs into application states, to support deep-linking and navigation.
 */
library angular2.router;

export "src/router/router.dart" show Router;
export "src/router/router_outlet.dart" show RouterOutlet;
export "src/router/router_link.dart" show RouterLink;
export "src/router/instruction.dart" show RouteParams;
export "src/router/route_registry.dart" show RouteRegistry;
export "src/router/location_strategy.dart" show LocationStrategy;
export "src/router/hash_location_strategy.dart" show HashLocationStrategy;
export "src/router/path_location_strategy.dart" show PathLocationStrategy;
export "src/router/location.dart" show Location, APP_BASE_HREF;
export "src/router/route_config_decorator.dart";
export "src/router/route_definition.dart";
export "src/router/interfaces.dart"
    show OnActivate, OnDeactivate, OnReuse, CanDeactivate, CanReuse;
export "src/router/lifecycle_annotations.dart" show CanActivate;
export "src/router/instruction.dart" show Instruction, ComponentInstruction;
export "package:angular2/angular2.dart" show OpaqueToken;
export "src/router/route_data.dart" show ROUTE_DATA;
import "src/router/location_strategy.dart" show LocationStrategy;
import "src/router/path_location_strategy.dart" show PathLocationStrategy;
import "src/router/router.dart" show Router, RootRouter;
import "src/router/router_outlet.dart" show RouterOutlet;
import "src/router/router_link.dart" show RouterLink;
import "src/router/route_registry.dart" show RouteRegistry;
import "src/router/location.dart" show Location;
import "core.dart" show bind, OpaqueToken, Binding;
import "src/core/facade/lang.dart" show Type;

/**
 * Token used to bind the component with the top-level [RouteConfig]s for the
 * application.
 *
 * You can use the [routerBindings] function in your [bootstrap] bindings to
 * simplify setting up these bindings.
 *
 * ## Example ([live demo](http://plnkr.co/edit/iRUP8B5OUbxCWQ3AcIDm))
 *
 * ```
 * import {Component, View} from 'angular2/angular2';
 * import {
 *   ROUTER_DIRECTIVES,
 *   ROUTER_BINDINGS,
 *   ROUTER_PRIMARY_COMPONENT,
 *   RouteConfig
 * } from 'angular2/router';
 *
 * @Component({...})
 * @View({directives: [ROUTER_DIRECTIVES]})
 * @RouteConfig([
 *  {...},
 * ])
 * class AppCmp {
 *   // ...
 * }
 *
 * bootstrap(AppCmp, [
 *   ROUTER_BINDINGS,
 *   bind(ROUTER_PRIMARY_COMPONENT).toValue(AppCmp)
 * ]);
 * ```
 */
const OpaqueToken ROUTER_PRIMARY_COMPONENT =
    const OpaqueToken("RouterPrimaryComponent");
/**
 * A list of directives. To use the router directives like [RouterOutlet] and
 * [RouterLink], add this to your `directives` array in the [View] decorator of your
 * component.
 *
 * ## Example ([live demo](http://plnkr.co/edit/iRUP8B5OUbxCWQ3AcIDm))
 *
 * ```
 * import {Component, View} from 'angular2/angular2';
 * import {ROUTER_DIRECTIVES, routerBindings, RouteConfig} from 'angular2/router';
 *
 * @Component({...})
 * @View({directives: [ROUTER_DIRECTIVES]})
 * @RouteConfig([
 *  {...},
 * ])
 * class AppCmp {
 *    // ...
 * }
 *
 * bootstrap(AppCmp, [routerBindings(AppCmp)]);
 * ```
 */
const List<dynamic> ROUTER_DIRECTIVES = const [RouterOutlet, RouterLink];
/**
 * A list of [Binding]s. To use the router, you must add this to your application.
 *
 * Note that you also need to bind to [ROUTER_PRIMARY_COMPONENT].
 *
 * You can use the [routerBindings] function in your [bootstrap] bindings to
 * simplify setting up these bindings.
 *
 * ## Example ([live demo](http://plnkr.co/edit/iRUP8B5OUbxCWQ3AcIDm))
 *
 * ```
 * import {Component, View} from 'angular2/angular2';
 * import {
 *   ROUTER_DIRECTIVES,
 *   ROUTER_BINDINGS,
 *   ROUTER_PRIMARY_COMPONENT,
 *   RouteConfig
 * } from 'angular2/router';
 *
 * @Component({...})
 * @View({directives: [ROUTER_DIRECTIVES]})
 * @RouteConfig([
 *  {...},
 * ])
 * class AppCmp {
 *   // ...
 * }
 *
 * bootstrap(AppCmp, [
 *   ROUTER_BINDINGS,
 *   bind(ROUTER_PRIMARY_COMPONENT).toValue(AppCmp)
 * ]);
 * ```
 */
const List<dynamic> ROUTER_BINDINGS = const [
  RouteRegistry,
  const Binding(LocationStrategy, toClass: PathLocationStrategy),
  Location,
  const Binding(Router,
      toFactory: routerFactory,
      deps: const [RouteRegistry, Location, ROUTER_PRIMARY_COMPONENT])
];
routerFactory(registry, location, primaryComponent) {
  return new RootRouter(registry, location, primaryComponent);
}

/**
 * A list of [Binding]s. To use the router, you must add these bindings to
 * your application.
 *
 * ## Example ([live demo](http://plnkr.co/edit/iRUP8B5OUbxCWQ3AcIDm))
 *
 * ```
 * import {Component, View} from 'angular2/angular2';
 * import {ROUTER_DIRECTIVES, routerBindings, RouteConfig} from 'angular2/router';
 *
 * @Component({...})
 * @View({directives: [ROUTER_DIRECTIVES]})
 * @RouteConfig([
 *  {...},
 * ])
 * class AppCmp {
 *   // ...
 * }
 *
 * bootstrap(AppCmp, [routerBindings(AppCmp)]);
 * ```
 */
List<dynamic> routerBindings(Type primaryComponent) {
  return [
    ROUTER_BINDINGS,
    bind(ROUTER_PRIMARY_COMPONENT).toValue(primaryComponent)
  ];
}
