library angular2.src.core.application_ref;

import "package:angular2/src/core/zone/ng_zone.dart" show NgZone;
import "package:angular2/src/core/facade/lang.dart"
    show Type, isBlank, isPresent, assertionsEnabled;
import "package:angular2/src/core/di.dart"
    show bind, Binding, Injector, OpaqueToken;
import "application_tokens.dart"
    show APP_COMPONENT_REF_PROMISE, APP_COMPONENT, APP_ID_RANDOM_BINDING;
import "package:angular2/src/core/facade/async.dart"
    show Future, PromiseWrapper, PromiseCompleter;
import "package:angular2/src/core/facade/collection.dart" show ListWrapper;
import "package:angular2/src/core/reflection/reflection.dart"
    show Reflector, reflector;
import "package:angular2/src/core/testability/testability.dart"
    show TestabilityRegistry, Testability;
import "package:angular2/src/core/linker/dynamic_component_loader.dart"
    show ComponentRef, DynamicComponentLoader;
import "package:angular2/src/core/facade/exceptions.dart"
    show BaseException, WrappedException, ExceptionHandler;
import "package:angular2/src/core/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/core/linker/view_ref.dart" show internalView;
import "package:angular2/src/core/life_cycle/life_cycle.dart" show LifeCycle;
import "package:angular2/src/core/change_detection/change_detection.dart"
    show
        IterableDiffers,
        defaultIterableDiffers,
        KeyValueDiffers,
        defaultKeyValueDiffers;
import "package:angular2/src/core/linker/view_pool.dart"
    show AppViewPool, APP_VIEW_POOL_CAPACITY;
import "package:angular2/src/core/linker/view_manager.dart" show AppViewManager;
import "package:angular2/src/core/linker/view_manager_utils.dart"
    show AppViewManagerUtils;
import "package:angular2/src/core/linker/view_listener.dart"
    show AppViewListener;
import "linker/proto_view_factory.dart" show ProtoViewFactory;
import "package:angular2/src/core/pipes.dart" show DEFAULT_PIPES;
import "linker/view_resolver.dart" show ViewResolver;
import "linker/directive_resolver.dart" show DirectiveResolver;
import "linker/pipe_resolver.dart" show PipeResolver;
import "package:angular2/src/core/linker/compiler.dart" show Compiler;

/**
 * Constructs the set of bindings meant for use at the platform level.
 *
 * These are bindings that should be singletons shared among all Angular applications
 * running on the page.
 */
List<dynamic /* Type | Binding | List < dynamic > */ > platformBindings() {
  return [bind(Reflector).toValue(reflector), TestabilityRegistry];
}

/**
 * Construct bindings specific to an individual root component.
 */
List<dynamic /* Type | Binding | List < dynamic > */ > _componentBindings(
    Type appComponentType) {
  return [
    bind(APP_COMPONENT).toValue(appComponentType),
    bind(APP_COMPONENT_REF_PROMISE).toFactory(
        (dynamicComponentLoader, Injector injector) {
      // TODO(rado): investigate whether to support bindings on root component.
      return dynamicComponentLoader
          .loadAsRoot(appComponentType, null, injector)
          .then((componentRef) {
        if (isPresent(componentRef.location.nativeElement)) {
          injector.get(TestabilityRegistry).registerApplication(
              componentRef.location.nativeElement, injector.get(Testability));
        }
        return componentRef;
      });
    }, [DynamicComponentLoader, Injector]),
    bind(appComponentType).toFactory(
        (Future<dynamic> p) => p.then((ref) => ref.instance),
        [APP_COMPONENT_REF_PROMISE])
  ];
}

/**
 * Construct a default set of bindings which should be included in any Angular
 * application, regardless of whether it runs on the UI thread or in a web worker.
 */
List<
    dynamic /* Type | Binding | List < dynamic > */ > applicationCommonBindings() {
  return [
    Compiler,
    APP_ID_RANDOM_BINDING,
    AppViewPool,
    bind(APP_VIEW_POOL_CAPACITY).toValue(10000),
    AppViewManager,
    AppViewManagerUtils,
    AppViewListener,
    ProtoViewFactory,
    ViewResolver,
    DEFAULT_PIPES,
    bind(IterableDiffers).toValue(defaultIterableDiffers),
    bind(KeyValueDiffers).toValue(defaultKeyValueDiffers),
    DirectiveResolver,
    PipeResolver,
    DynamicComponentLoader,
    bind(LifeCycle).toFactory(
        (exceptionHandler) => new LifeCycle(null, assertionsEnabled()),
        [ExceptionHandler])
  ];
}

/**
 * Create an Angular zone.
 */
NgZone createNgZone() {
  return new NgZone(enableLongStackTrace: assertionsEnabled());
}

PlatformRef _platform;
/**
 * @private
 */
PlatformRef platformCommon(
    [List<dynamic /* Type | Binding | List < dynamic > */ > bindings,
    dynamic /* () => void */ initializer]) {
  if (isPresent(_platform)) {
    if (isBlank(bindings)) {
      return _platform;
    }
    throw "platform() can only be called once per page";
  }
  if (isPresent(initializer)) {
    initializer();
  }
  if (isBlank(bindings)) {
    bindings = platformBindings();
  }
  _platform = new PlatformRef(Injector.resolveAndCreate(bindings), () {
    _platform = null;
  });
  return _platform;
}

/**
 * The Angular platform is the entry point for Angular on a web page. Each page
 * has exactly one platform, and services (such as reflection) which are common
 * to every Angular application running on the page are bound in its scope.
 *
 * A page's platform is initialized implicitly when [bootstrap]() is called, or
 * explicitly by calling [platform]().
 */
class PlatformRef {
  Injector _injector;
  dynamic /* () => void */ _dispose;
  /**
   * @private
   */
  List<ApplicationRef> _applications = [];
  /**
   * @private
   */
  PlatformRef(this._injector, this._dispose) {}
  /**
   * Retrieve the platform [Injector], which is the parent injector for
   * every Angular application on the page and provides singleton bindings.
   */
  Injector get injector {
    return this._injector;
  }

  /**
   * Instantiate a new Angular application on the page.
   *
   * # What is an application?
   *
   * Each Angular application has its own zone, change detection, compiler,
   * renderer, and other framework components. An application hosts one or more
   * root components, which can be initialized via `ApplicationRef.bootstrap()`.
   *
   * # Application Bindings
   *
   * Angular applications require numerous bindings to be properly instantiated.
   * When using `application()` to create a new app on the page, these bindings
   * must be provided. Fortunately, there are helper functions to configure
   * typical bindings, as shown in the example below.
   *
   * # Example
   * ```
   * var myAppBindings = [MyAppService];
   *
   * platform()
   *   .application([applicationCommonBindings(), applicationDomBindings(), myAppBindings])
   *   .bootstrap(MyTopLevelComponent);
   * ```
   * # See Also
   *
   * See the [bootstrap] documentation for more details.
   */
  ApplicationRef application(
      List<dynamic /* Type | Binding | List < dynamic > */ > bindings) {
    var app = this._initApp(createNgZone(), bindings);
    return app;
  }

  /**
   * Instantiate a new Angular application on the page, using bindings which
   * are only available asynchronously. One such use case is to initialize an
   * application running in a web worker.
   *
   * # Usage
   *
   * `bindingFn` is a function that will be called in the new application's zone.
   * It should return a [Promise] to a list of bindings to be used for the
   * new application. Once this promise resolves, the application will be
   * constructed in the same manner as a normal `application()`.
   */
  Future<ApplicationRef> asyncApplication(
      dynamic /* (zone: NgZone) =>
                       Promise<Array<Type | Binding | any[]>> */
      bindingFn) {
    var zone = createNgZone();
    var completer = PromiseWrapper.completer();
    zone.run(() {
      PromiseWrapper.then(bindingFn(zone),
          (List<dynamic /* Type | Binding | List < dynamic > */ > bindings) {
        completer.resolve(this._initApp(zone, bindings));
      });
    });
    return completer.promise;
  }

  ApplicationRef _initApp(NgZone zone,
      List<dynamic /* Type | Binding | List < dynamic > */ > bindings) {
    Injector injector;
    zone.run(() {
      bindings.add(bind(NgZone).toValue(zone));
      bindings.add(bind(ApplicationRef).toValue(this));
      var exceptionHandler;
      try {
        injector = this.injector.resolveAndCreateChild(bindings);
        exceptionHandler = injector.get(ExceptionHandler);
        zone.overrideOnErrorHandler((e, s) => exceptionHandler.call(e, s));
      } catch (e, e_stack) {
        if (isPresent(exceptionHandler)) {
          exceptionHandler.call(e, e_stack);
        } else {
          DOM.logError(e);
        }
      }
    });
    var app = new ApplicationRef(this, zone, injector);
    this._applications.add(app);
    return app;
  }

  /**
   * Destroy the Angular platform and all Angular applications on the page.
   */
  void dispose() {
    this._applications.forEach((app) => app.dispose());
    this._dispose();
  }

  /**
   * @private
   */
  void _applicationDisposed(ApplicationRef app) {
    ListWrapper.remove(this._applications, app);
  }
}

/**
 * A reference to an Angular application running on a page.
 *
 * For more about Angular applications, see the documentation for [bootstrap].
 */
class ApplicationRef {
  PlatformRef _platform;
  NgZone _zone;
  Injector _injector;
  List<Function> _bootstrapListeners = [];
  List<ComponentRef> _rootComponents = [];
  /**
   * @private
   */
  ApplicationRef(this._platform, this._zone, this._injector) {}
  /**
   * Register a listener to be called each time `bootstrap()` is called to bootstrap
   * a new root component.
   */
  void registerBootstrapListener(
      dynamic /* (ref: ComponentRef) => void */ listener) {
    this._bootstrapListeners.add(listener);
  }

  /**
   * Bootstrap a new component at the root level of the application.
   *
   * # Bootstrap process
   *
   * When bootstrapping a new root component into an application, Angular mounts the
   * specified application component onto DOM elements identified by the [componentType]'s
   * selector and kicks off automatic change detection to finish initializing the component.
   *
   * # Optional Bindings
   *
   * Bindings for the given component can optionally be overridden via the `bindings`
   * parameter. These bindings will only apply for the root component being added and any
   * child components under it.
   *
   * # Example
   * ```
   * var app = platform.application([applicationCommonBindings(), applicationDomBindings()];
   * app.bootstrap(FirstRootComponent);
   * app.bootstrap(SecondRootComponent, [bind(OverrideBinding).toClass(OverriddenBinding)]);
   * ```
   */
  Future<ComponentRef> bootstrap(Type componentType,
      [List<dynamic /* Type | Binding | List < dynamic > */ > bindings]) {
    var completer = PromiseWrapper.completer();
    this._zone.run(() {
      var componentBindings = _componentBindings(componentType);
      if (isPresent(bindings)) {
        componentBindings.add(bindings);
      }
      var exceptionHandler = this._injector.get(ExceptionHandler);
      try {
        Injector injector =
            this._injector.resolveAndCreateChild(componentBindings);
        Future<ComponentRef> compRefToken =
            injector.get(APP_COMPONENT_REF_PROMISE);
        var tick = (componentRef) {
          var appChangeDetector =
              internalView(componentRef.hostView).changeDetector;
          var lc = injector.get(LifeCycle);
          lc.registerWith(this._zone, appChangeDetector);
          lc.tick();
          completer.resolve(componentRef);
          this._rootComponents.add(componentRef);
          this
              ._bootstrapListeners
              .forEach((listener) => listener(componentRef));
        };
        var tickResult = PromiseWrapper.then(compRefToken, tick);
        PromiseWrapper.then(tickResult, (_) {});
        PromiseWrapper.then(tickResult, null,
            (err, stackTrace) => completer.reject(err, stackTrace));
      } catch (e, e_stack) {
        exceptionHandler.call(e, e_stack);
        completer.reject(e, e_stack);
      }
    });
    return completer.promise;
  }

  /**
   * Retrieve the application [Injector].
   */
  Injector get injector {
    return this._injector;
  }

  /**
   * Retrieve the application [NgZone].
   */
  NgZone get zone {
    return this._zone;
  }

  /**
   * Dispose of this application and all of its components.
   */
  void dispose() {
    // TODO(alxhub): Dispose of the NgZone.
    this._rootComponents.forEach((ref) => ref.dispose());
    this._platform._applicationDisposed(this);
  }
}
