// Some of the code comes from WebComponents.JS

// https://github.com/webcomponents/webcomponentsjs/blob/master/src/HTMLImports/path.js
library angular2.src.core.compiler.style_url_resolver;

import "package:angular2/src/core/facade/lang.dart"
    show RegExp, RegExpWrapper, StringWrapper, isPresent;
import "package:angular2/src/core/compiler/url_resolver.dart" show UrlResolver;

/**
 * Rewrites URLs by resolving '@import' and 'url()' URLs from the given base URL,
 * removes and returns the @import urls
 */
StyleWithImports resolveStyleUrls(
    UrlResolver resolver, String baseUrl, String cssText) {
  var foundUrls = [];
  cssText = extractUrls(resolver, baseUrl, cssText, foundUrls);
  cssText = replaceUrls(resolver, baseUrl, cssText);
  return new StyleWithImports(cssText, foundUrls);
}

class StyleWithImports {
  String style;
  List<String> styleUrls;
  StyleWithImports(this.style, this.styleUrls) {}
}

String extractUrls(UrlResolver resolver, String baseUrl, String cssText,
    List<String> foundUrls) {
  return StringWrapper.replaceAllMapped(cssText, _cssImportRe, (m) {
    var url = isPresent(m[1]) ? m[1] : m[2];
    foundUrls.add(resolver.resolve(baseUrl, url));
    return "";
  });
}

String replaceUrls(UrlResolver resolver, String baseUrl, String cssText) {
  return StringWrapper.replaceAllMapped(cssText, _cssUrlRe, (m) {
    var pre = m[1];
    var originalUrl = m[2];
    if (RegExpWrapper.test(_dataUrlRe, originalUrl)) {
      // Do not attempt to resolve data: URLs
      return m[0];
    }
    var url = StringWrapper.replaceAll(originalUrl, _quoteRe, "");
    var post = m[3];
    var resolvedUrl = resolver.resolve(baseUrl, url);
    return pre + "'" + resolvedUrl + "'" + post;
  });
}

var _cssUrlRe = new RegExp(r'(url\()([^)]*)(\))');
var _cssImportRe = new RegExp(r'@import\s+(?:url\()?\s*(?:(?:[' +
    "'" +
    r'"]([^' +
    "'" +
    r'"]*))|([^;\)\s]*))[^;]*;?');
var _quoteRe = new RegExp(r'[' + "'" + r'"]');
var _dataUrlRe = new RegExp(r'^[' + "'" + r'"]?data:');
