library angular2.src.core.directives.ng_style;

import "package:angular2/lifecycle_hooks.dart" show DoCheck;
import "package:angular2/src/core/change_detection.dart"
    show KeyValueDiffer, KeyValueDiffers;
import "package:angular2/src/core/linker.dart" show ElementRef;
import "package:angular2/src/core/metadata.dart" show Directive;
import "package:angular2/src/core/render.dart" show Renderer;
import "package:angular2/src/core/facade/lang.dart"
    show isPresent, isBlank, print;

/**
 * Adds or removes styles based on an {expression}.
 *
 * When the expression assigned to `ng-style` evaluates to an object, the corresponding element
 * styles are updated. Style names to update are taken from the object keys and values - from the
 * corresponding object values.
 *
 * # Example:
 *
 * ```
 * <div [ng-style]="{'text-align': alignExp}"></div>
 * ```
 *
 * In the above example the `text-align` style will be updated based on the `alignExp` value
 * changes.
 *
 * # Syntax
 *
 * - `<div [ng-style]="{'text-align': alignExp}"></div>`
 * - `<div [ng-style]="styleExp"></div>`
 */
@Directive(selector: "[ng-style]", inputs: const ["rawStyle: ng-style"])
class NgStyle implements DoCheck {
  KeyValueDiffers _differs;
  ElementRef _ngEl;
  Renderer _renderer;
  var _rawStyle;
  KeyValueDiffer _differ;
  NgStyle(this._differs, this._ngEl, this._renderer) {}
  set rawStyle(v) {
    this._rawStyle = v;
    if (isBlank(this._differ) && isPresent(v)) {
      this._differ = this._differs.find(this._rawStyle).create(null);
    }
  }

  doCheck() {
    if (isPresent(this._differ)) {
      var changes = this._differ.diff(this._rawStyle);
      if (isPresent(changes)) {
        this._applyChanges(changes);
      }
    }
  }

  void _applyChanges(dynamic changes) {
    changes.forEachAddedItem((record) {
      this._setStyle(record.key, record.currentValue);
    });
    changes.forEachChangedItem((record) {
      this._setStyle(record.key, record.currentValue);
    });
    changes.forEachRemovedItem((record) {
      this._setStyle(record.key, null);
    });
  }

  void _setStyle(String name, String val) {
    this._renderer.setElementStyle(this._ngEl, name, val);
  }
}
