library angular2.src.mock.directive_resolver_mock;

import "package:angular2/src/core/facade/collection.dart"
    show Map, MapWrapper, ListWrapper;
import "package:angular2/src/core/facade/lang.dart"
    show Type, isPresent, stringify, isBlank, print;
import "../core/metadata.dart" show DirectiveMetadata, ComponentMetadata;
import "package:angular2/src/core/linker/directive_resolver.dart"
    show DirectiveResolver;

class MockDirectiveResolver extends DirectiveResolver {
  var _bindingsOverrides = new Map<Type, List<dynamic>>();
  var _viewBindingsOverrides = new Map<Type, List<dynamic>>();
  DirectiveMetadata resolve(Type type) {
    var dm = super.resolve(type);
    var bindingsOverride = this._bindingsOverrides[type];
    var viewBindingsOverride = this._viewBindingsOverrides[type];
    var bindings = dm.bindings;
    if (isPresent(bindingsOverride)) {
      bindings = (new List.from(dm.bindings)..addAll(bindingsOverride));
    }
    if (dm is ComponentMetadata) {
      var viewBindings = dm.viewBindings;
      if (isPresent(viewBindingsOverride)) {
        viewBindings = (new List.from(dm.viewBindings)
          ..addAll(viewBindingsOverride));
      }
      return new ComponentMetadata(
          selector: dm.selector,
          inputs: dm.inputs,
          outputs: dm.outputs,
          host: dm.host,
          bindings: bindings,
          exportAs: dm.exportAs,
          moduleId: dm.moduleId,
          queries: dm.queries,
          changeDetection: dm.changeDetection,
          viewBindings: viewBindings);
    }
    return new DirectiveMetadata(
        selector: dm.selector,
        inputs: dm.inputs,
        outputs: dm.outputs,
        host: dm.host,
        bindings: bindings,
        exportAs: dm.exportAs,
        moduleId: dm.moduleId,
        queries: dm.queries);
  }

  void setBindingsOverride(Type type, List<dynamic> bindings) {
    this._bindingsOverrides[type] = bindings;
  }

  void setViewBindingsOverride(Type type, List<dynamic> viewBindings) {
    this._viewBindingsOverrides[type] = viewBindings;
  }
}
