library angular2.src.web_workers.worker.application_common;

import "package:angular2/src/core/di.dart"
    show Injector, bind, OpaqueToken, Binding;
import "package:angular2/src/core/forms.dart" show FORM_BINDINGS;
import "package:angular2/src/core/facade/lang.dart"
    show
        NumberWrapper,
        Type,
        isBlank,
        isPresent,
        assertionsEnabled,
        print,
        stringify;
import "package:angular2/src/core/facade/exceptions.dart" show ExceptionHandler;
import "package:angular2/src/core/facade/async.dart"
    show Future, PromiseWrapper, PromiseCompleter;
import "package:angular2/src/core/compiler/xhr.dart" show XHR;
import "package:angular2/src/web_workers/worker/xhr_impl.dart"
    show WebWorkerXHRImpl;
import "package:angular2/src/core/compiler/app_root_url.dart" show AppRootUrl;
import "renderer.dart" show WebWorkerRenderer;
import "package:angular2/src/core/render/api.dart" show Renderer;
import "package:angular2/src/web_workers/shared/client_message_broker.dart"
    show ClientMessageBrokerFactory;
import "package:angular2/src/web_workers/shared/message_bus.dart"
    show MessageBus;
import "package:angular2/src/core/application_ref.dart"
    show platformCommon, PlatformRef, ApplicationRef, applicationCommonBindings;
import "package:angular2/src/web_workers/shared/serializer.dart"
    show Serializer;
import "package:angular2/src/web_workers/shared/api.dart" show ON_WEB_WORKER;
import "package:angular2/src/web_workers/shared/render_proto_view_ref_store.dart"
    show RenderProtoViewRefStore;
import "package:angular2/src/web_workers/shared/render_view_with_fragments_store.dart"
    show RenderViewWithFragmentsStore;
import "package:angular2/src/core/facade/async.dart" show ObservableWrapper;
import "package:angular2/src/web_workers/shared/messaging_api.dart"
    show SETUP_CHANNEL;
import "package:angular2/src/web_workers/worker/event_dispatcher.dart"
    show WebWorkerEventDispatcher;
import "package:angular2/src/core/linker/dynamic_component_loader.dart"
    show ComponentRef;
import "package:angular2/src/core/zone/ng_zone.dart" show NgZone;
import "package:angular2/src/core/compiler/compiler.dart" show compilerBindings;

/**
 * Initialize the Angular 'platform' on the page in a manner suitable for applications
 * running in a web worker. Applications running on a web worker do not have direct
 * access to DOM APIs.
 *
 * See [PlatformRef] for details on the Angular platform.
 *
 * # Without specified bindings
 *
 * If no bindings are specified, `platform`'s behavior depends on whether an existing
 * platform exists:
 *
 * If no platform exists, a new one will be created with the default [platformBindings].
 *
 * If a platform already exists, it will be returned (regardless of what bindings it
 * was created with). This is a convenience feature, allowing for multiple applications
 * to be loaded into the same platform without awareness of each other.
 *
 * # With specified bindings
 *
 * It is also possible to specify bindings to be made in the new platform. These bindings
 * will be shared between all applications on the page. For example, an abstraction for
 * the browser cookie jar should be bound at the platform level, because there is only one
 * cookie jar regardless of how many applications on the age will be accessing it.
 *
 * If bindings are specified directly, `platform` will create the Angular platform with
 * them if a platform did not exist already. If it did exist, however, an error will be
 * thrown.
 *
 * # For Web Worker Appplications
 *
 * This version of `platform` initializes Angular for use with applications
 * that do not directly touch the DOM, such as applications which run in a
 * web worker context. Applications that need direct access to the DOM should
 * use `platform` from `core/application_common` instead.
 */
PlatformRef platform(
    [List<dynamic /* Type | Binding | List < dynamic > */ > bindings]) {
  return platformCommon(bindings);
}

class PrintLogger {
  var log = print;
  var logError = print;
  var logGroup = print;
  logGroupEnd() {}
}

List<dynamic /* Type | Binding | List < dynamic > */ > webWorkerBindings(
    appComponentType, MessageBus bus, Map<String, dynamic> initData) {
  return [
    compilerBindings(),
    Serializer,
    bind(MessageBus).toValue(bus),
    ClientMessageBrokerFactory,
    WebWorkerRenderer,
    bind(Renderer).toAlias(WebWorkerRenderer),
    bind(ON_WEB_WORKER).toValue(true),
    RenderViewWithFragmentsStore,
    RenderProtoViewRefStore,
    bind(ExceptionHandler)
        .toFactory(() => new ExceptionHandler(new PrintLogger()), []),
    WebWorkerXHRImpl,
    bind(XHR).toAlias(WebWorkerXHRImpl),
    bind(AppRootUrl).toValue(new AppRootUrl(initData["rootUrl"])),
    WebWorkerEventDispatcher,
    FORM_BINDINGS
  ];
}

Future<ComponentRef> bootstrapWebWorkerCommon(
    Type appComponentType, MessageBus bus,
    [List<dynamic /* Type | Binding | List < dynamic > */ > appBindings =
        null]) {
  PromiseCompleter<dynamic> bootstrapProcess = PromiseWrapper.completer();
  var appPromise = platform().asyncApplication((NgZone zone) {
    // TODO(rado): prepopulate template cache, so applications with only

    // index.html and main.js are possible.

    //
    bus.attachToZone(zone);
    bus.initChannel(SETUP_CHANNEL, false);
    dynamic subscription;
    var emitter = bus.from(SETUP_CHANNEL);
    subscription = ObservableWrapper.subscribe(emitter,
        (Map<String, dynamic> message) {
      var bindings = [
        applicationCommonBindings(),
        webWorkerBindings(appComponentType, bus, message)
      ];
      if (isPresent(appBindings)) {
        bindings.add(appBindings);
      }
      bootstrapProcess.resolve(bindings);
      ObservableWrapper.dispose(subscription);
    });
    ObservableWrapper.callNext(bus.to(SETUP_CHANNEL), "ready");
    return bootstrapProcess.promise;
  });
  return PromiseWrapper.then(
      appPromise, (app) => app.bootstrap(appComponentType));
}
