library angular2.test.core.pipes.pipes_spec;

import "package:angular2/test_lib.dart"
    show
        ddescribe,
        xdescribe,
        describe,
        it,
        iit,
        xit,
        expect,
        beforeEach,
        afterEach;
import "package:angular2/core.dart"
    show Injector, Inject, bind, Pipe, PipeTransform;
import "package:angular2/src/core/pipes/pipes.dart" show ProtoPipes, Pipes;
import "package:angular2/src/core/pipes/pipe_binding.dart" show PipeBinding;

class PipeA implements PipeTransform {
  transform(a, b) {}
  onDestroy() {}
}

class PipeB implements PipeTransform {
  var dep;
  PipeB(@Inject("dep") dynamic dep) {
    this.dep = dep;
  }
  transform(a, b) {}
  onDestroy() {}
}

main() {
  describe("Pipes", () {
    var injector;
    beforeEach(() {
      injector = Injector.resolveAndCreate([bind("dep").toValue("dependency")]);
    });
    it("should instantiate a pipe", () {
      var proto = ProtoPipes.fromBindings(
          [PipeBinding.createFromType(PipeA, new Pipe(name: "a"))]);
      var pipes = new Pipes(proto, injector);
      expect(pipes.get("a").pipe).toBeAnInstanceOf(PipeA);
    });
    it("should throw when no pipe found", () {
      var proto = ProtoPipes.fromBindings([]);
      var pipes = new Pipes(proto, injector);
      expect(() => pipes.get("invalid"))
          .toThrowErrorWith("Cannot find pipe 'invalid'");
    });
    it("should inject dependencies from the provided injector", () {
      var proto = ProtoPipes.fromBindings(
          [PipeBinding.createFromType(PipeB, new Pipe(name: "b"))]);
      var pipes = new Pipes(proto, injector);
      expect(((pipes.get("b").pipe as dynamic)).dep).toEqual("dependency");
    });
    it("should cache pure pipes", () {
      var proto = ProtoPipes.fromBindings(
          [PipeBinding.createFromType(PipeA, new Pipe(name: "a", pure: true))]);
      var pipes = new Pipes(proto, injector);
      expect(pipes.get("a").pure).toEqual(true);
      expect(pipes.get("a")).toBe(pipes.get("a"));
    });
    it("should NOT cache impure pipes", () {
      var proto = ProtoPipes.fromBindings([
        PipeBinding.createFromType(PipeA, new Pipe(name: "a", pure: false))
      ]);
      var pipes = new Pipes(proto, injector);
      expect(pipes.get("a").pure).toEqual(false);
      expect(pipes.get("a")).not.toBe(pipes.get("a"));
    });
  });
}
